@@ -4,8 +4,9 @@ Functions to implement the v0 One Codex API calls.
 import json
 import os
 import requests
-from requests_toolbelt import MultipartEncoder
+from requests_toolbelt import MultipartEncoder, MultipartEncoderMonitor
 import sys
+from multiprocessing import Lock, Value
 from threading import BoundedSemaphore, Thread
 import urlparse
 from onecodex import version
@@ -21,7 +22,7 @@ else:
 BASE_URL = urlparse.urlparse(BASE_API)
 BASE_URL = BASE_URL._replace(path='/').geturl()
 DEFAULT_THREADS = 4
-
+CHUNK_SIZE = 8192
 
 BAD_AUTH_MSG = ("\nYour login credentials appear be bad. Try logging out:"
                 "\n    onecodex logout"
@@ -66,6 +67,36 @@ def get_update_message():
             print j["message"]
 
 
+def upload_callback(monitor, upload_progress_bytes, lock, total_bytes, n_files):
+    if upload_progress_bytes.value == -1:
+        return
+    with lock:
+        upload_progress_bytes.value += CHUNK_SIZE  # Chunk size
+    if upload_progress_bytes.value == 0:
+        progress = 0.0
+    else:
+        progress = upload_progress_bytes.value / float(total_bytes)
+    # print upload_progress_bytes.value, monitor.bytes_read
+    barLength = 20  # Modify this to change the length of the progress bar
+    if progress < 0:
+        progress = 0
+        status = "Halt...                       \r\n"  # Needs to be longer than files string
+    elif progress >= 1:
+        progress = 1
+        status = "Done.                         \r\n"
+        with lock:
+            upload_progress_bytes.value = -1
+    else:
+        status = ("(%d files remaining)" % n_files.value
+                  if n_files.value > 1
+                  else "(1 file remaining)")
+    block = int(round(barLength * progress))
+    text = "\rUploading: [{0}] {1:.2f}% {2}".format("#" * block + "-" * (barLength - block),
+                                                    progress * 100, status)
+    sys.stdout.write(text)
+    sys.stdout.flush()
+
+
 # Upload functions
 def upload(args):
     """
@@ -94,16 +125,23 @@ def upload(args):
     callback_url = BASE_URL.rstrip("/") + j0['callback_url']
 
     upload_threads = []
+    upload_progress_bytes = Value('i', 0)
+    upload_progress_lock = Lock()
+    total_bytes = sum([os.path.getsize(f) for f in args.file])
+    total_files = Value('i', len(args.file))
     for f in args.file:
-        if args.threads:  # parallel uploads
+        if args.threads and len(args.file) > 1:  # parallel uploads
             # Multi-threaded uploads
             t = Thread(target=upload_helper,
                        args=(f, s3_url, signing_url, callback_url,
-                             creds, semaphore))
+                             creds, upload_progress_bytes, upload_progress_lock,
+                             total_bytes, total_files, semaphore))
             upload_threads.append(t)
             t.start()
         else:  # serial uploads
-            upload_helper(f, s3_url, signing_url, callback_url, creds)
+            upload_helper(f, s3_url, signing_url, callback_url, creds,
+                          upload_progress_bytes, upload_progress_lock,
+                          total_bytes, total_files)
 
     if args.threads:
         for ut in upload_threads:
@@ -111,6 +149,8 @@ def upload(args):
 
 
 def upload_helper(f, s3_url, signing_url, callback_url, creds,
+                  upload_progress_bytes, upload_progress_lock,
+                  total_bytes, total_files,
                   semaphore=None):
     # First get the signing form data
     if semaphore is not None:
@@ -130,7 +170,11 @@ def upload_helper(f, s3_url, signing_url, callback_url, creds,
         fields.append((str(k), str(v)))
 
     fields.append(("file", (stripped_filename, open(f, mode='rb'), "text/plain")))
-    m = MultipartEncoder(fields)
+    e = MultipartEncoder(fields)
+    m = MultipartEncoderMonitor(e, lambda x: upload_callback(x, upload_progress_bytes,
+                                                             upload_progress_lock,
+                                                             total_bytes=(total_bytes + 8192),
+                                                             n_files=total_files))
     r2 = requests.post(s3_url, data=m, headers={"Content-Type": m.content_type})
     if r2.status_code != 201:
         print "Upload failed. Please contact help@onecodex.com for assistance."
@@ -142,8 +186,11 @@ def upload_helper(f, s3_url, signing_url, callback_url, creds,
         "size": os.path.getsize(f)
     })
     if r3.status_code == 200:
+        if upload_progress_bytes.value > 0:  # == -1 upon completion
+            print "\r"
         print "Successfully uploaded: %s" % f
-        pass
+        with upload_progress_lock:
+            total_files.value -= 1
     else:
         print "Failed to upload: %s" % f
         sys.exit(1)
